#!/usr/bin/bash

# PATH_to_tp.sh -- convert all *.c files in the directoy tree of invocation from PATH to taged comments.

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


# bring dsp281x/dsp281x*.c files into top evel so maker can find them
cp dsp281x/dsp281x*.[ch] .
cp IQmath/*.[ch] .

# make a list of all the files we want to process
###find . -name '*.[ch]' -print | awk '{ print substr($0,3) }' > files.dat
ls *.[ch] > files.dat

# strip trailing CR's and trailing blanks
for f in `cat files.dat`
do cat $f | ./tbf.sh > $f.tbf
done

# put a flagpole on block comments
for f in `cat files.dat`
do awk 'BEGIN       { flag = 0 }
        /\*\//      { flag = 0 }
        flag == 1   { print " * " $0 }
        flag == 0   { print $0 }
        $1 ~ "/*"   { if (substr($1,1,2) == "/*" && $0 !~ "*/") { flag = 1 }}' $f.tbf > $f.boxed
done

# tranform PATH macro calls to tagged comments
for f in `cat files.dat`
do awk 'substr($1,1,5) == "PATH(" { c = index($1,",") ; 
                                    p1 = substr($1,6,c-6) ; 
                                    p2 = substr($1,c+1) ;
                                    p2 = substr(p2,1,index(p2,")")-1) ;
                                    print " /* \\tp Path_Coverage -->",p1,p2,"*/" ;
                                    next }
                                  { print $0 }' $f.boxed > $f.tagged
done

# convert // comments to /* comments */
for f in `cat files.dat`
do cat $f \
    | sed 's.///.//.g' \
    | awk '/^\ \*/ { print $0 ; next }
           /\/\//  { ss = index($0,"//")
                     print substr($0,1,ss-1) " /* " substr($0,ss+2) " */" 
                     next }
                   { print $0 }' $f.tagged > $f.ansi
done

# fix // comments that ended with */ before we converted them.  :-(
for f in `cat files.dat`
do cat $f.ansi \
    | sed 's.\*/ \*/.\*/.g' > $f.noduh
done

# properly indent the resulting source code
for f in `cat files.dat`
do 
   indent -npsl $f.noduh -o $f.indented
done

# move the final result into its own directory tree
rm -rf cleaned_up
mkdir -p cleaned_up/
mkdir -p cleaned_up/IQmath
mkdir -p cleaned_up/build
mkdir -p cleaned_up/dsp281x

# copy files into cleaned_up directory tree
for f in `cat files.dat`
do cp $f.indented cleaned_up/$f 2>/dev/null
done

# crock fix-up for #include "..\parameter.h" problem
for f in dsp281x.c dsp281x_pie.c
do cat $f.indented | sed 's/\.\.\\//g' > cleaned_up/$f
done

# clean up temporary files
rm -f files.dat *.tbf *.boxed *.tagged *.ansi *.noduh *.indented

cp maker.sh cflow2dep.pl recursive_depends_on.pl dot_makefile call_graph.sh include_graph.sh all_nodes_in.sh clickable_subgraphs.sh dir.sh dir.head dir.foot subgraph_containing.sh recursive_depends_on.sh recursive_refers_to.sh reverse_direction.sh find_sink_nodes.sh find_source_nodes.sh dep.foot dep.head dep.sty empty.html index.html cleaned_up/
cd cleaned_up

./maker.sh foo
make clean
make

./include_graph.sh

./call_graph.sh

