#!/usr/bin/perl

# cflow2dep.pl -- filter to convert cflow output into dep input.

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


# cflow outputs a file with 4 leading spaces to indent according to
# each level of call nesting, then the function name with parens, then
# the full signature in angle brackets, followed by a colon unless no
# further functions calls are to be made, in which case the colon s
# absent.  Example:

#    Motor_Update() <void Motor_Update (Motor_t *p) at motor.c:340>:
#        Timer_SetTime() <bool Timer_SetTime (Timer_t *timer,Uint16 flags,Uint32 duration) at timer.c:401>:

# Each function that calls another function needs a dependency line in
# the output file.  This line is a quote string containing the calling
# function name, then the unquoted dependency operator, then a quoted
# string containing the called function name, then a terminating
# semicolon.  Example:

# "Motor_Update" -> "Timer_SetTime";

# It is the job of this script to convert the output og cflow into a
# file containing these dependency lines.

my $lin = "";						     # the current input line

my $next = 0;						     # the current level of nexting
my $old_nest = 0;					     # the previous level of nesting

my @stack;						     # the call stack

while(<>) {						     # for each line of the input
  chomp;

  $lin = substr($_,0,index($_,"("));			     # throw away parens and signature

  for ($nest = 0; substr($lin,0,4) eq "    "; $nest++) {     # determine nesting and discard leading spaces
    $lin = substr($lin,4);
  }

  $stack[$nest] = $lin;					     # insert function name into call stack

  print "$stack[$nest-1] -> $stack[$nest]\n" if $nest > 0;   # write the output line
}
