#!/usr/bin/bash

# clickable_subgraphs.sh -- make clickable subgraphs for all subgraphs in last graph

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


export PATH=$PATH:.

# preserve the dependency and style files from the big graph
cat dep.dot | tr -d '";' > big.dot
cp dep.sty big.sty

# make a list of all the nodes in the big graph
cat big.dot | all_nodes_in.sh > clickable_subgraphs.nodes

for node in `cat clickable_subgraphs.nodes`
do echo $node

   # make a list of dependencies for the subgraph containing $node
   subgraph_containing.sh big.dot $node > subgraph.dot

   # make a list of the nodes in that subgraph
   cat subgraph.dot | all_nodes_in.sh > subgraph.nodes

   # surround each node name with quotes
   awk -v dq='"' '{ print dq $0 dq }' subgraph.nodes > quoted.nodes

   # sort the big graph style file and purge duplicates
   cat big.sty | sort -u > sorted.sty

   # select the style lines that apply
   join sorted.sty quoted.nodes > nodes.sty

   # make the selected node stand out visually
   cat nodes.sty \
    | awk -v n=$node 'substr($1,2,length($1)-2) == n { print substr($0,1,index($0,"];")-1) ",peripheries=7];" ; next }
                                                     { print $0 }' - \
   > subgraph.sty

   # pass the dependency and style files to the makefile
   cp subgraph.dot dep.dot
   cp subgraph.sty dep.sty

   # generate the pdf file for the graph
   make -f dot_makefile clean > /dev/null
   make -f dot_makefile 2>&1 > /dev/null

   # put the result in the right place
   cp dep.pdf $node.pdf
   cp dep.ps $node.ps
   cp dep.in $node.dot

done

