#!/bin/bash

#      maker <executable> <optimization> -- make a makefile

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


# This script generates a makefile for the C program comprised of all
# the *.c files in the current directory.  It does this by running the
# Gnu C compiler on the directory's *.c files to generate make style
# dependency statements.  These are wrapped up in statements to set
# the Gnu C compiler and its options, and to link all the *.o files
# produced by the *.c files into the target.  This should
# automatically make a makefile that will make the program for most
# typical portable Gnu C Unix application programs.


executable=$1
optimization=$2

rm -f Makefile

echo '# This Makefile was AUTOMATICALLY GENERATED by "maker' $executable $optimization'"' >> Makefile
echo '# for' `whoami`@`hostname`:`pwd` >> Makefile
echo '# on' `date` >> Makefile
echo '' >> Makefile

C=`ls *.c | tr '\n' ' '`
H=`ls *.h | tr '\n' ' '`
S=`echo $C | sed 's/\.c/\.s/g'   | sed 's/\.cc/\.s/g' \
           | sed 's/\.cpp/\.s/g' | sed 's/\.c/\.s/g'`
L=`echo $C | sed 's/\.c/\.lst/g'   | sed 's/\.cc/\.lst/g' \
           | sed 's/\.cpp/\.lst/g' | sed 's/\.c/\.lst/g'`
O=`echo $C | sed 's/\.c/\.o/g'   | sed 's/\.cc/\.o/g' \
           | sed 's/\.cpp/\.o/g' | sed 's/\.c/\.o/g'`

echo '######## Macros:' >> Makefile
echo '#' >> Makefile
echo LISTING=listnew >> Makefile
echo CC=gcc >> Makefile
echo 'CFLAGS= -g -O'$optimization >> Makefile
INC=''
echo INC="$INC" >> Makefile
echo 'LIB=-lm -lc' >> Makefile

echo SOURCES=$C >> Makefile
echo HEADERS=$H >> Makefile
echo ASMS=$S >> Makefile
echo OBJECTS=$O >> Makefile

echo '' >> Makefile

echo '######## Rules:' >> Makefile
echo '#' >> Makefile

echo '%.o: %.s' >> Makefile
echo '	$(CC) $(CFLAGS) $(INC) -Wa,-alhcn -c $*.s > $*.lst' >> Makefile
echo ''  >> Makefile
echo '%.s: %.c' >> Makefile
echo '	$(CC) $(CFLAGS) $(INC) -S $*.c' >> Makefile

echo '' >> Makefile

echo '######## Targets:' >> Makefile
echo '#' >> Makefile
echo all : TAGS $executable >> Makefile
echo "" >> Makefile

echo '' >> Makefile

echo clean : >> Makefile
echo '	rm -f TAGS' $executable '$(OBJECTS) $(ASMS)' >> Makefile

echo '' >> Makefile

echo "$executable : \$(OBJECTS)"  >> Makefile
echo "	\$(CC) -g -o $executable \$(OBJECTS) \$(LIB)" >> Makefile

echo '' >> Makefile

echo 'TAGS : $(C) $(H)' >> Makefile
echo "	etags -S \$(SOURCES) \$(HEADERS)" >> Makefile

echo '' >> Makefile

echo '######## Dependencies:' >> Makefile
echo '#' >> Makefile

gcc -MM $INC $C | sed s/\.o\:/\.s\:/g >> Makefile
shift

