#!/usr/bin/bash

# module_graph.sh -- graph object module dependencies

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


echo '---------------- module graphs ----------------'

# default to local mode
MODE=${MODE:-"file:"}
if [ "$MODE" != "file:" ]
then MODE=${MODE}/module_graph/
fi

# get rid of any old results
rm -rf module_graph
mkdir -p module_graph

# start with a clean slate
rm -f *.pdf *.ps *.dot

# list all global defs and refs
nm -C -o -g *.o \
 | awk '{ file = substr($1, 1, index($1, ":")-3) ".c"
          type = $2
          x = index($3, "(")
          if (x == 0) { symbol = $3 }
          else { symbol = substr($3, 1, x-1) }
          print file, type, symbol }' - \
> nm.out

# make a list of all the undefineds
grep '\ U\ ' nm.out | grep -v nesting | sort -k 3,3 -u > nm.U

# make a list of everything but the undefineds
grep -v '\ U\ ' nm.out | grep -v nesting | sort -k 3,3 -u > nm.D

# join them on the symbol name
join -j 3 nm.U nm.D \
 | awk '{print $2,$4}' - \
 | sort -u \
 | awk -v dq='"' '{ print dq $1 dq, "->", dq $2 dq }' - > module_graph.dot
cp module_graph.dot dep.dot

# make all files cyan hexagons
cat nm.out \
 | awk -v mode=$MODE -v dq='"' '{ print dq $1 dq, "[shape=hexagon,style=filled,fillcolor=lightskyblue,href=" dq mode  $1 ".pdf" dq "];" }' - \
 | sort -u > module_graph.sty
cp module_graph.sty dep.sty

# as a result of the above, symbols that are still undefined will be white elipses!

# generate the graph
make -f dot_makefile clean > /dev/null
make -f dot_makefile 2>&1 > /dev/null
mv dep.pdf '==ALL==.pdf'
mv dep.ps '==ALL==.ps'
mv dep.in '==ALL==.dot'

# generate all of the subgraphs
./extern_subgraphs.sh

# now make the table of contents
./module_graph_dir.sh > dir.html

# put in final home
#cp *.html *.pdf *.png *.gif module_graph/
cp *.html *.pdf *.gif module_graph/

# fix up title for browser
cat index.html.skel | sed 's/Dependency Graph Browser/Module Graph/' > module_graph/index.html

