#!/usr/bin/bash

# subgraph_containing.sh <dbase> <node_name> -- generate depends list
# on stdout for the subgraph of <dbase> that contains <node_name>

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


# fetch parameters
dbase=$1
node_name=$2

# clean nasty quotes and semicolons out of dbase!  :-(
cat $dbase | tr -d '";' > $dbase.dequoted
dbase=$dbase.dequoted

# get what's below it
./recursive_depends_on.sh $dbase $node_name > subgraph_containing.dup

# get what's above it
./recursive_refers_to.sh  $dbase $node_name >> subgraph_containing.dup

# purge duplicates to get minimal list
cat subgraph_containing.dup \
 | tr -d ';' \
 | awk '{ print $1 ; print $3 }' - \
 | sort -u \
> subgraph_containing.min

# Select only those nodes from the input <dbase> file.  This means
# that only nodes in the subgraph_containing.min list are allowed to
# appear in the result.  Every node in the LHS must be on this list,
# as must every node of the RHS.

# strip funky chars from the input file and sort on LHS
cat $dbase \
 | tr -d '";' \
 | sort -u \
> subgraph_containing.db1

# extract all lines in <dbase> whose LHS matches a line in list of nodes in the subgraph
join subgraph_containing.db1 subgraph_containing.min > subgraph_containing.lhs

# do it again for the RHS

# first, re-sort on RHS
cat subgraph_containing.lhs \
 | sort -k 3 \
> subgraph_containing.db2

# then match RHS against list of nodes in the subgraph
join -1 3 -o 1.1,1.2,1.3 subgraph_containing.db2 subgraph_containing.min > subgraph_containing.rhs

# re-install quotes around node names
awk -v dq='"' '{ print dq $1 dq,$2,dq $3 dq }' subgraph_containing.rhs

