#!/usr/bin/perl

# tarjan.pl -- inputs a graph and outputs the subgraph of its strongly connected components, 1 SCC per line

# ----------------------------------------------------------------
# This file is part of code_graphs Copyright (C) 2011 Elijah 
# Laboratories Inc.  http://www.elilabs.com  ALL RIGHTS RESERVED.
#
# GNU cflow is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the
# Free Software Foundation; either version 3 of the License, or (at your
# option) any later version.
#
# code_graphs is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with code_graphs; if not, write to the Free Software Foundation,
# Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
# ----------------------------------------------------------------


# This is Tarjan's "Strongly Connected Components Algorithm", as
# described in Algorithms in Java, Third Edition,Pg. 174, Program
# 19.11.  Robert Sedgewick, Addison-Wesley Professional, 2003.  This
# Perl version was translated from the Java with adaptations for use
# in the code_graphs call_graph.sh script.


$udef = -1;				      # undefined indicator

$cnt = 0;				      # counts nodes in the order in which they are traversed
					      # by depth first search (DFS)

$scnt = 0;				      # counts strongly connected components (SCC's)

$G_V = 0;				      # counts number of nodes in graph

%id = ();				      # keeps track of which SCC a node is in
%pre = ();				      # the number of this node DFS search order
%low = ();				      # the number of the first node of the current SCC
					      # encountered in DFS order

%suc = ();				      # list of successors of a node

@S = ();				      # the stack where nodes are accumulated


sub input {				      # read the edges from stdin
  while (<>) {				      # read a line
    chomp;
    next unless /->/;			      # is it an edge?

    ($src,$arrow,$dst)=split;		      # yes, get the source and destination node names

    $pre{$src} = $udef;			      # initialize the source node of this edge
    $low{$src} = $udef;
    $id{$src} = $udef;

    $pre{$dst} = $udef;			      # initialize the destination node of this edge
    $low{$dst} = $udef;
    $id{$dst} = $udef;

    push @{$suc{$src}}, ($dst);		      # append the destination node name to the successor list

    $G_V++;				      # count the node
  }
  return;
}


sub strong_connect {			      # find a strongly connected component
  my $w = shift(@_);
  my $t = $udef;

  my $min = $cnt;
  $low{$w} = $cnt;
  $pre{$w} = $cnt;
  $cnt++;

  push @S, ($w);

  foreach $t (@{$suc{$w}}) {		      # Consider successors of v
    strong_connect($t) if $pre{$t} == $udef; # if successor w has not yet been visited; recurse on it
    $min = $low{$t} if $low{$t} < $min;
  }

  if ($min < $low{$w}) {
    $low{$w} = $min;

    return;
  }

  for (;;) {				      # best I could do in Perl for a do...while loop  :-(
    $t = pop @S;
    $id{$t} = $scnt;
    $low{$t} = $G_V;
    last if $t eq $w;
  }
  $scnt++;

  return;
}


### main program starts here ####

input();				      # input

foreach $v (keys %pre) {		      # process
  strong_connect($v) if $pre{$v} == $udef;
}

###print "\n";
foreach $v (keys %pre) {		      # output
  print "$id{$v} $v\n";
}

### end ###

